/*
 * Copyright (C) 2014 digitalSTROM.org, Schlieren, Switzerland
 *
 * digitalSTROM ds485-base library is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * digitalSTROM ds485-base library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with digitalSTROM ds485-base library. If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef __BASE_DS_PACKET_H__
#define __BASE_DS_PACKET_H__

#include "include/ds485n-packet.h"

#include <stdbool.h>

/**
 * header of ds485n_packet_t and ds485_container differs
 * - ds485n_packet_t contains dummy field to ensure alignment
 * - ds485n_packet_t has 16bit length ds485_container only 8bit
 * hence we can not declare a common header struct and
 * cast the container/pointer to that type
 *
 * either we clutter the code with (type == v_container) tests
 * everywhere or we cache the fields and copy them back before
 * before forwarding the containers
 * CAUTION: always call ds485b_pu_sync after doing changes
 */
typedef struct {
    dsuid_t src;
    dsuid_t dst;
    unsigned char transactionId;
    uint32_t containerType;

    enum {
        v_container,
        v_packet
    } type;

    union {
        const ds485_container_t *container;
        const ds485n_packet_t *packet;
    } u;
} packet_union_t;

void ds485b_pu_from_short(packet_union_t *pkt, const ds485_container_t *cont);
void ds485b_pu_from_long(packet_union_t *dst, const ds485n_packet_t *src);

/**
 * write modified src/transactionId back to either short or long container
 * see description in packet_untiont declaration
 */
void ds485b_pu_sync(packet_union_t *packet);

/**
 * @ret true if size does not exceed upper bound
 */
bool ds485b_pu_is_oversize(const packet_union_t *packet);

#endif
