#include "lib_simple_client.h"

#include <netinet/ip.h>
#include <check.h>

#include "include/ds485-socket-interface.h"

#include "base/socket_send.h"

int connect_server(int port)
{
    struct sockaddr_in dest;
    int s, ret;

    /* no cloexec, seems testcase will be forked */
    s = socket(AF_INET, SOCK_STREAM, 0);

    dest.sin_family = AF_INET;
    dest.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
    dest.sin_port = htons(port);

    ret = connect(s, (struct sockaddr *) &dest, sizeof(dest));
    if (ret < 0) {
        ck_abort_msg("failed to connect");
    }
    return s;
}

int send_container(int s, ds485_container_t *pkt)
{
    struct iovec iov[2];
    char buf[3];
    int length;

    length = DS485N_TLV_HDR_LEN + DS485N_CONT_LEN(pkt);
    buf[0] = DS485D_CMD_TX_PACKET;
    buf[1] = length & 0xff;
    buf[2] = (length >> 8) & 0xff;

    iov[0].iov_base = buf;
    iov[0].iov_len = DS485N_TLV_HDR_LEN;
    iov[1].iov_base = pkt;
    iov[1].iov_len = DS485N_CONT_LEN(pkt);

    return socket_sendv(s, iov, 2);
}

int recv_container(int s, ds485_container_t *pkt)
{
    struct msghdr mh;
    struct iovec iov[2];
    char tlv[3];

    iov[0].iov_base = tlv;
    iov[0].iov_len = DS485N_TLV_HDR_LEN;
    iov[1].iov_base = pkt;
    iov[1].iov_len = sizeof(*pkt);

    memset(&mh, 0, sizeof(mh));
    mh.msg_iovlen = 2;
    mh.msg_iov = iov;

    return recvmsg(s, &mh, MSG_DONTWAIT);
}
